@@ -27,25 +27,35 @@
 #include <locale.h>
 
 #include <QDebug>
-#include <QtTest/QtTest>
+#include <QtTest>
 
 #include <libintl.h>
 
 #include <klocalizedstring.h>
 
-#include <QtCore/QSet>
-#include <QtCore/QString>
+#include <QSet>
+#include <QString>
 
 void KLocalizedStringTest::initTestCase()
 {
     KLocalizedString::setApplicationDomain("ki18n-test");
 
     m_hasFrench = true;
+    m_hasCatalan = true;
+
+    setlocale(LC_ALL, "ca_ES.utf8");
+    if (setlocale(LC_ALL, nullptr) != QByteArray("ca_ES.utf8")) {
+        qDebug() << "Failed to set locale to ca_ES.utf8.";
+        m_hasCatalan = false;
+    }
+
     if (m_hasFrench) {
         setlocale(LC_ALL, "fr_FR.utf8");
         if (setlocale(LC_ALL, nullptr) != QByteArray("fr_FR.utf8")) {
             qDebug() << "Failed to set locale to fr_FR.utf8.";
             m_hasFrench = false;
+        } else {
+            QLocale::setDefault(QLocale("fr_FR")); // the setlocale is "too late" for Qt that already has created the default QLocale, so set it manually
         }
     }
     if (m_hasFrench) {
@@ -56,7 +66,10 @@ void KLocalizedStringTest::initTestCase()
     }
     QDir dataDir(m_tempDir.path());
     if (m_hasFrench) {
-        m_hasFrench = compileCatalogs({QFINDTESTDATA("po/fr/ki18n-test.po"), QFINDTESTDATA("po/fr/ki18n-test-qt.po")}, dataDir);
+        m_hasFrench = compileCatalogs({QFINDTESTDATA("po/fr/ki18n-test.po"), QFINDTESTDATA("po/fr/ki18n-test-qt.po")}, dataDir, "fr");
+    }
+    if (m_hasCatalan) {
+        m_hasCatalan = compileCatalogs({QFINDTESTDATA("po/ca/ki18n-test.po")}, dataDir, "ca");
     }
     if (m_hasFrench) {
         qputenv("XDG_DATA_DIRS",
@@ -76,9 +89,10 @@ void KLocalizedStringTest::initTestCase()
 #endif
 }
 
-bool KLocalizedStringTest::compileCatalogs(const QStringList &testPoPaths, const QDir &dataDir)
+bool KLocalizedStringTest::compileCatalogs(const QStringList &testPoPaths, const QDir &dataDir, const QString &lang)
 {
-    if (!dataDir.mkpath("locale/fr/LC_MESSAGES")) {
+    const QString lcMessages = QString("locale/%1/LC_MESSAGES").arg(lang);
+    if (!dataDir.mkpath(lcMessages)) {
         qDebug() << "Failed to create locale subdirectory "
                  "inside temporary directory.";
         return false;
@@ -88,13 +102,13 @@ bool KLocalizedStringTest::compileCatalogs(const QStringList &testPoPaths, const
         qDebug() << "msgfmt(1) not found in path.";
         return false;
     }
-    foreach (const QString &testPoPath, testPoPaths) {
+    for (const QString &testPoPath : testPoPaths) {
         int pos_1 = testPoPath.lastIndexOf(QLatin1Char('/'));
         int pos_2 = testPoPath.lastIndexOf(QLatin1Char('.'));
         QString domain = testPoPath.mid(pos_1 + 1, pos_2 - pos_1 - 1);
         QString testMoPath;
-        testMoPath = QString::fromLatin1("%1/locale/fr/LC_MESSAGES/%2.mo")
-                     .arg(dataDir.path(), domain);
+        testMoPath = QString::fromLatin1("%1/%3/%2.mo")
+                     .arg(dataDir.path(), domain, lcMessages);
         QProcess process;
         QStringList arguments;
         arguments << testPoPath << QLatin1String("-o") << testMoPath;
@@ -111,6 +125,9 @@ bool KLocalizedStringTest::compileCatalogs(const QStringList &testPoPaths, const
 
 void KLocalizedStringTest::correctSubs()
 {
+    if (!m_hasFrench) {
+        QSKIP("French test files not usable.");
+    }
     // Warm up.
     QCOMPARE(i18n("Daisies, daisies"),
              QString("Daisies, daisies"));
@@ -207,7 +224,7 @@ void KLocalizedStringTest::correctSubs()
     QCOMPARE(ki18n("%1").subs(42, -5, 10, QChar('_')).toString(),
              QString("42___"));
     QCOMPARE(ki18n("%1").subs(4.2, 5, 'f', 2).toString(),
-             QString(" 4.20"));
+             QString(" 4,20"));
 }
 
 void KLocalizedStringTest::wrongSubs()
@@ -235,6 +252,8 @@ void KLocalizedStringTest::wrongSubs()
 
 void KLocalizedStringTest::semanticTags()
 {
+    KLocalizedString::setLanguages({"en"});
+
     // <application/>
     QCOMPARE(xi18nc("@action:inmenu", "Open with <application>%1</application>", "Okteta"),
              QString("Open with Okteta"));
@@ -436,6 +455,9 @@ void KLocalizedStringTest::miscMethods()
         QSet<QString> availableLanguages;
         availableLanguages.insert("fr");
         availableLanguages.insert("en_US");
+        if (m_hasCatalan) {
+            availableLanguages.insert("ca");
+        }
         QCOMPARE(KLocalizedString::availableApplicationTranslations(),
                  availableLanguages);
     }
@@ -465,6 +487,7 @@ void KLocalizedStringTest::translateToFrench()
     QCOMPARE(i18n("Job"), QString::fromUtf8("Tâche"));
 }
 
+#if KI18N_ENABLE_DEPRECATED_SINCE(5, 0)
 void KLocalizedStringTest::translateQt()
 {
     KLocalizedString::insertQtDomain("ki18n-test-qt");
@@ -487,6 +510,7 @@ void KLocalizedStringTest::translateQt()
 #endif
     KLocalizedString::removeQtDomain("ki18n-test-qt");
 }
+#endif
 
 void KLocalizedStringTest::testLocalizedTranslator()
 {
@@ -519,7 +543,7 @@ void KLocalizedStringTest::addCustomDomainPath()
         QSKIP("French test files not usable.");
     }
     QTemporaryDir dir;
-    compileCatalogs({QFINDTESTDATA("po/fr/ki18n-test2.po")}, dir.path());
+    compileCatalogs({QFINDTESTDATA("po/fr/ki18n-test2.po")}, dir.path(), "fr");
     KLocalizedString::addDomainLocaleDir("ki18n-test2", dir.path() + "/locale");
 
     QSet<QString> expectedAvailableTranslations({"en_US", "fr"});
@@ -527,6 +551,40 @@ void KLocalizedStringTest::addCustomDomainPath()
     QCOMPARE(i18nd("ki18n-test2", "Cheese"), QString::fromUtf8("Fromage"));
 }
 
+void KLocalizedStringTest::multipleLanguages()
+{
+    if (!m_hasFrench || !m_hasCatalan) {
+        QSKIP("French or Catalan test files not usable.");
+    }
+    KLocalizedString::setLanguages({"ca"});
+    QCOMPARE(i18n("Job"), QString::fromUtf8("Job")); // This is not the actual catalan translation but who cares
+    KLocalizedString::setLanguages({"fr"});
+    QCOMPARE(i18n("Job"), QString::fromUtf8("Tâche"));
+    KLocalizedString::setLanguages({"ca", "fr"});
+    QCOMPARE(i18n("Job"), QString::fromUtf8("Job")); // This is not the actual catalan translation but who cares
+
+
+    KLocalizedString::setLanguages({"ca"});
+    QCOMPARE(i18n("Loadable modules"), QString::fromUtf8("Loadable modules")); // The po doesn't have a translation so we get the English text
+    KLocalizedString::setLanguages({"fr"});
+    QCOMPARE(i18n("Loadable modules"), QString::fromUtf8("Modules chargeables"));
+    KLocalizedString::setLanguages({"ca", "fr"});
+    QCOMPARE(i18n("Loadable modules"), QString::fromUtf8("Modules chargeables")); // The Catalan po doesn't have a translation so we get the English text
+}
+
+void KLocalizedStringTest::untranslatedText()
+{
+    if (!m_hasFrench) {
+        QSKIP("French test files not usable.");
+    }
+    KLocalizedString s = ki18n("Job");
+    KLocalizedString::setLanguages({"fr"});
+    QCOMPARE(s.untranslatedText(), "Job");
+    QCOMPARE(s.toString(), QString::fromUtf8("Tâche"));
+    QCOMPARE(s.untranslatedText(), "Job");
+}
+
+
 #include <QThreadPool>
 #include <QtConcurrentRun>
 #include <QFutureSynchronizer>
@@ -538,8 +596,10 @@ void KLocalizedStringTest::testThreads()
     sync.addFuture(QtConcurrent::run(this, &KLocalizedStringTest::correctSubs));
     sync.addFuture(QtConcurrent::run(this, &KLocalizedStringTest::correctSubs));
     sync.addFuture(QtConcurrent::run(this, &KLocalizedStringTest::correctSubs));
+#if KI18N_ENABLE_DEPRECATED_SINCE(5, 0)
     sync.addFuture(QtConcurrent::run(this, &KLocalizedStringTest::translateQt));
     sync.addFuture(QtConcurrent::run(this, &KLocalizedStringTest::translateQt));
+#endif
     sync.addFuture(QtConcurrent::run(this, &KLocalizedStringTest::translateToFrench));
     sync.waitForFinished();
     QThreadPool::globalInstance()->setMaxThreadCount(1); // delete those threads
